import React from 'react';
import {
  colorPaletteConfigSlice,
  querybuilderSlice,
  setQuerybuilderNodes,
} from '@graphpolaris/shared/lib/data-access/store';
import { GraphPolarisThemeProvider } from '@graphpolaris/shared/lib/data-access/theme';
import { configureStore } from '@reduxjs/toolkit';
import { Meta, ComponentStory } from '@storybook/react';
import { Provider } from 'react-redux';
import QueryBuilder from './querybuilder';
import { MultiGraph } from 'graphology';
import {
  addPill,
  handles,
} from '@graphpolaris/shared/lib/querybuilder/usecases';

const Component: Meta<typeof QueryBuilder> = {
  component: QueryBuilder,
  title: 'Panels/QueryBuilder',
  decorators: [
    (story) => (
      <Provider store={mockStore}>
        <GraphPolarisThemeProvider>{story()}</GraphPolarisThemeProvider>
      </Provider>
    ),
  ],
};

// Mock palette store
const mockStore = configureStore({
  reducer: {
    colorPaletteConfig: colorPaletteConfigSlice.reducer,
    querybuilder: querybuilderSlice.reducer,
  },
});
const graph = new MultiGraph();
addPill('0', { type: 'entity', x: 100, y: 100, name: 'Entity Pill' }, graph);
// graph.addNode('0', { type: 'entity', x: 100, y: 100, name: 'Entity Pill' });
addPill(
  '1',
  { type: 'relation', x: 140, y: 140, name: 'Relation Pill' },
  graph
);
addPill(
  '2',
  {
    type: 'attribute',
    x: 170,
    y: 160,
    name: 'Attr string',
    datatype: 'string',
    operator: 'EQ',
    value: 'mark',
  },
  graph
);
addPill(
  '3',
  {
    type: 'attribute',
    x: 170,
    y: 170,
    name: 'Attr number',
    datatype: 'float',
    operator: 'EQ',
  },
  graph
);
addPill(
  '4',
  {
    type: 'attribute',
    x: 130,
    y: 120,
    name: 'Attr bool',
    datatype: 'bool',
    operator: 'EQ',
    value: 'true',
  },
  graph
);
console.log(graph.getNodeAttributes('2'));
graph.addEdge('2', '1', { type: 'attribute_connection' });
graph.addEdge('3', '1', { type: 'attribute_connection' });
graph.addEdge('4', '0', { type: 'attribute_connection' });
graph.addEdge('0', '1', {
  type: 'entity_relation',
  targetHandle: handles.relation.fromEntity,
});
// graph.addEdge('1', '0', {
//   type: 'entity_relation',
//   sourceHandle: handles.relation.entity,
// });
mockStore.dispatch(setQuerybuilderNodes(graph.export()));

export const Simple = {
  args: {},
};

export default Component;
